/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
*     National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
*     Operator of Los Alamos National Laboratory.
* EPICS BASE is distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution. 
\*************************************************************************/

/* drvTS.h */

/**************************************************************************
 *
 *     Author:	Jim Kowalkowski
 *
 ***********************************************************************/

#ifndef INC_drvTS_H
#define INC_drvTS_H

#include "epicsTypes.h"
#include "shareLib.h"

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/

/*
 * Network packets from the timing master
 */

#define TS_MAGIC ('T'<<24|'S'<<16|'d'<<8|'r')
#define TS_SLAVE_PORT  18322
#define TS_MASTER_PORT 18323
#define TS_TIME_OUT_MS 250
#define TS_STAMP_FROM_UNIX 0x0001

typedef enum {
    TS_time_request,
    TS_sync_request,
    TS_sync_msg
} TStype;

typedef struct TSstampTransStruct {
    epicsUInt32 magic;             /* identifier */
    epicsUInt32 type;              /* transaction type */
    struct timespec master_time;   /* master time stamp - last sync time */
    struct timespec current_time;  /* master current time stamp 1990 epoch */
    struct timespec unix_time;     /* time using 1970 epoch */
    epicsUInt32 sync_rate;         /* master sends sync at this rate */
    epicsUInt32 clock_hz;          /* master clock this frequency (tick rate) */
    /* Fields below here were introduced in 3.14.x */
    epicsUInt32 mask;              /* extra info */
    epicsUInt32 syncReportThreshold;     /*for sync errors*/
    epicsUInt32 syncUnixReportThreshold; /*for TS_STAMP_FROM_UNIXerrors*/
} TSstampTrans;


/*
 * Event system and time clock functions
 */

typedef void (*TSeventFunc)(int Card, int EventNum, epicsUInt32 Ticks);
typedef void (*TSerrorFunc)(int Card, int ErrorNum);

struct drvTSops {
    TSeventFunc (*TSregisterEventHandler)(int Card, TSeventFunc func);
    TSerrorFunc (*TSregisterErrorHandler)(int Card, TSerrorFunc func);
    long (*TSgetTicks)(int Card, epicsUInt32 *Ticks);
    long (*TShaveReceiver)(int Card);
    long (*TSforceSync)(int Card);
    long (*TSgetTime)(struct timespec*);
    long (*TSsyncEvent)();
    long (*TSdirectTime)();
    long (*TSdriverInit)();
    long (*TSuserGet)(int event_number,struct timespec* sp);
};

epicsShareFunc void drvTsRegisterDeviceSupport(struct drvTSops *ops);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* INC_drvTS_H */
